#!/bin/sh

# Installs or uninstalls (if -u option is specified) custom pixbuf loader
# Place this script in the directory where the loader (checkboard.so) is.

# Get the directory where this script resides.
WORK_DIR=$(cd `dirname $0` && pwd) 
cd ${WORK_DIR}

PATH=/opt/gnome/bin:$PATH

CUSTOM_MODULE=checkboard.so
file ${CUSTOM_MODULE} | grep '32-bit' >/dev/null
if [ $? -eq 0 ]
then
#        echo "Attempting to find and update the list of 32-bit loaders..."
        QL_CMD=gdk-pixbuf-query-loaders
        LDR_32_BIT="yes"
        LDR_LIB="lib"
else
#        echo "Attempting to find and update the list of 64-bit loaders..."
        QL_CMD=gdk-pixbuf-query-loaders-64
        LDR_32_BIT=""
        LDR_LIB="lib64"
fi

which ${QL_CMD} >/dev/null 2>&1
if [ $? -ne 0 ]
then
    QL_CMD=gdk-pixbuf-query-loaders

    which ${QL_CMD} >/dev/null 2>&1
    if [ $? -ne 0 ]
    then
        QL_CMD=gdk-pixbuf-query-loaders-32
    fi
fi 

# Find appropriate .loaders file to update
if [ -z ${GDK_PIXBUF_MODULE_FILE} ]
then
    if [ ! -d "${SYSCONFDIR}/gtk-2.0" ]
    then
        SYSCONFDIR=/etc
        if [ ! -d "${SYSCONFDIR}/gtk-2.0" ]  
        then
            SYSCONFDIR=/etc/opt/gnome
            if [ ! -d "${SYSCONFDIR}/gtk-2.0" ]  
            then
                echo "Unable to locate directory containing the .loaders file."
                exit 1
            fi
        fi
    fi

    LFILES=`find ${SYSCONFDIR}/gtk-2.0/ -name '*.loaders*'`
    LFILES_NONL=`echo ${LFILES} | tr -d \n`

    if [ -z "${LFILES_NONL}" ]; then
#        echo "Unable to locate the .loaders file in ${SYSCONFDIR}/gtk-2.0/ or its subdirectories."
#        exit 1
# This fallback action may help to circumvent the issues on Ubuntu and 
# similar systems where .loaders file is not present by default.
        LFILES="${SYSCONFDIR}/gtk-2.0/gdk-pixbuf.loaders"
        touch "${LFILES}" || exit 1
    fi
    
    which uname >/dev/null 2>&1 
    if [ $? -eq 0 ]; then
        # uname command is available
        ARCHNAME=`uname -m`
        GDK_PIXBUF_MODULE_FILE=`echo "${LFILES}" | grep ${ARCHNAME}`
    else
        # uname command is not available
        GDK_PIXBUF_MODULE_FILE=""
    fi
    
    if [ -z "${GDK_PIXBUF_MODULE_FILE}" ]; then
        # .loaders file has been found but not in a directory that has
        # architecture string in its name
        # If the loader is a 64-bit module, look for the the files with "64"
        # in their names.
        if [ -z "$LDR_32_BIT" ]; then    
            GDK_PIXBUF_MODULE_FILE=`echo "${LFILES}" | grep "64" | tail -1`
        fi
        # ia64 doesn't fit the above model
        if [ -z "$LDR_32_BIT" -a "$ARCHNAME" = "ia64" ]; then
	    GDK_PIXBUF_MODULE_FILE=`echo "${LFILES}" | grep -v "64" | tail -1`
	fi
        # on a dual-arch system in 32 bit mode, where both loaders and 
        # loaders64 happen to be present, the uname test above clears the 
        # list out, then the last (if nothing else) stanza below plugs in 
        # loaders64, which is wrong
        if [ -n "$LDR_32_BIT" ]; then
            GDK_PIXBUF_MODULE_FILE=`echo "${LFILES}" | grep -v "64" | tail -1`
        fi
        # If nothing of the above helps to identify appropriate .loaders-file,
        # use the last file in the list.
        if [ -z "${GDK_PIXBUF_MODULE_FILE}" ]; then
            GDK_PIXBUF_MODULE_FILE=`echo "${LFILES}" | tail -1`
        fi
    fi
fi

# newer systems put things in /usr/lib{64}/gdk-pixbuf-2.0/
# see bug 2455
# and to make things interesting, some put the command under bin there
CMD_DIR=/usr/${LDR_LIB}/gdk-pixbuf-2.0
CACHE_DIR=/usr/${LDR_LIB}/gdk-pixbuf-2.0/2.10.0
QL_CMD_NEW=gdk-pixbuf-query-loaders

# and now, we have things moving to /usr/lib/$(ARCH)-linux-gnu on 
# some systems, try one more attempt to get at the right file
# Note: dpkg-architecture/DEB_HOST_MULTIARCH is the correct way on
# Debian based systems, but use a fallback
which dpkg-architecture >/dev/null 2>&1 
if [ $? -eq 0 ]; then
    eval `dpkg-architecture | grep DEB_HOST_MULTIARCH`
else
    if [ "$ARCHNAME" = "i486" -o "$ARCHNAME" = "i586" -o "$ARCHNAME" = "i686" ];then
        DEB_HOST_MULTIARCH="i386-linux-gnu"
    else
        DEB_HOST_MULTIARCH="$ARCHNAME-linux-gnu"
    fi
fi

if [ ! -f "$CACHE_DIR/loaders.cache" ]; then
    CMD_DIR=/usr/lib/${DEB_HOST_MULTIARCH}/gdk-pixbuf-2.0
    CACHE_DIR=/usr/lib/${DEB_HOST_MULTIARCH}/gdk-pixbuf-2.0/2.10.0
fi

if [ -x "${CMD_DIR}/${QL_CMD_NEW}" ]; then
    QL_CMD=${CMD_DIR}/${QL_CMD_NEW}
    GDK_PIXBUF_MODULE_FILE=${CACHE_DIR}/loaders.cache
fi
if [ -x "${CMD_DIR}/bin/${QL_CMD_NEW}" ]; then
    QL_CMD=${CMD_DIR}/bin/${QL_CMD_NEW}
    GDK_PIXBUF_MODULE_FILE=${CACHE_DIR}/loaders.cache
fi

# binary is on PATH, but loaders.cache setup
which ${QL_CMD} >/dev/null 2>&1
if [ $? -eq 0 ];then
    if [ -f "${CACHE_DIR}/loaders.cache" ];then
        GDK_PIXBUF_MODULE_FILE=${CACHE_DIR}/loaders.cache
    fi
fi

echo "Updating the list of image loaders in ${GDK_PIXBUF_MODULE_FILE}"

if [ "$1" = '-u' ] 
then
#uninstall
#    echo "Uninstalling custom pixbuf loader..."
    ${QL_CMD} > ${GDK_PIXBUF_MODULE_FILE}
else
#install
#    echo "Installing custom pixbuf loader..."
    ${QL_CMD} ${CUSTOM_MODULE} >> ${GDK_PIXBUF_MODULE_FILE} 
fi
