#!/bin/sh

# T2C_ROOT is not necessary to run the tests

# Get the directory where this script resides.
WORK_DIR=$(cd `dirname $0` && pwd) 
T2C_SUITE_ROOT=${WORK_DIR}
export T2C_SUITE_ROOT

cd ${T2C_SUITE_ROOT}

if [ -z ${TET_ROOT} ]
then
    TET_ROOT=/opt/lsb-tet3-lite
fi

echo TET_ROOT is ${TET_ROOT}
#echo T2C_SUITE_ROOT is ${T2C_SUITE_ROOT}

export TET_ROOT
export TET_PATH=$TET_ROOT
export PATH=/opt/lsb/bin:$PATH:$TET_PATH/bin:/sbin
export TET_SUITE_ROOT=`pwd`

export CC=lsbcc
for x in /opt/lsb/lib*/pkgconfig; do
    PKG_CONFIG_PATH="${x}${PKG_CONFIG_PATH+:${PKG_CONFIG_PATH}}"
done
export PKG_CONFIG_PATH

if [ "$1" = "-noxvfb" ]; then
    USE_XVFB=
    shift
else
    USE_XVFB=yes
fi

XT_DISPLAYHOST=localhost
XT_FONTPATH=/opt/lsb/share/fonts/misc/ 

# Find a free display to use for Xvfb

DISPLAY_NUM=9          #default display number
HOOK_DISPLAY_NUM=25    #default display number for hooking

if [ "${USE_XVFB}" = "yes" ]; then

    which xprop > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        #OK, found xprop, now use it to get a free display number

        DNUM_LIST="8 7 6 5 4 3 2 9 10 11 12" #try these display numbers only
        
        for DNUM in `echo ${DNUM_LIST}`; do
        xprop -display :${DNUM}.0 -root >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            # no properties found for this display, guess it is free
            DISPLAY_NUM=${DNUM}
            break
        fi
        done

        #now look for a free display for hooking
        DHOOK_NUM_LIST="25 26 27 28 29 30 24 23"    
        for DHOOK in `echo ${DHOOK_NUM_LIST}`; do
        xprop -display :${DHOOK}.0 -root >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            # no properties found for this display, guess it is free
            HOOK_DISPLAY_NUM=${DHOOK}
            break
        fi
        done

    fi

fi

export XT_DISPLAY=":${DISPLAY_NUM}.0"
export XT_HOOK_DISPLAY=":${HOOK_DISPLAY_NUM}.0"
export TEST_DISPLAY=${XT_DISPLAY}
echo "Display for hooking: ${XT_HOOK_DISPLAY} will be used."

# XT_LOG_LEVEL - This may be set to a debugging level between 0 and 3.
# If  bVerbose is set to 0, this value has no effect.
# 0 - produce no output, only fatal errors will be shown
# 1 - produce error output from LOG_ERROR
# 2 - produce information output from LOG_INFO
# 3 - produce debugging output from LOG_DEBUG
export XT_LOG_LEVEL=3 

XVFB_RUNNING=""
#---------------------------------------------------------
# start xvfb if it isn't already running
startXvfb()
{
    running=`pidof Xvfb`
    XVFB_RUNNING=${running}
    if [ -z "${running}" ];then
        XVFB_PATH=/opt/lsb/bin/Xvfb
        if [ ! -x  ${XVFB_PATH} ]; then
            echo "ERROR: ${XVFB_PATH} is missing"
            exit 1
        fi
    
        ${XVFB_PATH} -render -screen 0 1024x768x24 $TEST_DISPLAY -ac +bs -kb -fp $XT_FONTPATH >/dev/null 2>&1 &
        echo "Starting X Virtual Frame Buffer"; sleep 20
        
        running_after=`pidof Xvfb`
        if [ -z "${running_after}" ];then
            echo "ERROR: failed to start Xvfb"
            exit 1
        fi
        
        export DISPLAY=${TEST_DISPLAY}
        export G_SLICE=always-malloc
        export XT_DISPLAY
        echo "Display ${XT_DISPLAY} will be used for Xvfb."
    else
        #Xvfb is running, determine the display number
        CMD_XVFB=`ps -p "${running}" -f | tail -n 1`
        export XT_DISPLAY=`echo $CMD_XVFB | sed 's/.*\(:[0-9]*\.0\).*/\1/'`
        export DISPLAY=${XT_DISPLAY}
        export G_SLICE=always-malloc
        
        echo "Xvfb is already running (display: ${DISPLAY}), so it will be used."
    fi

}

stopXvfb()
{
    if [ -z "${XVFB_RUNNING}" ]; then
        running=`pidof Xvfb`
    else
        running=""
    fi

    if [ -n "$running" ];then
        kill $running
    fi
}

if [ ! -z "${USE_XVFB}" ]; then
    startXvfb
fi

#---------------------------------------------------------
installmimetype()
{
    XDG_MIME=`command -v xdg-mime`
    if [ -n "${XDG_MIME}" ];then
        echo "Installing mime type for checkboard test image..."
        ${XDG_MIME} install mime-types/lsb-test-desktop-t2c-checkboard.xml
    else
        echo "Unable to find xdg-mime command, additional test errors possible..."
    fi
}

removemimetype()
{
    XDG_MIME=`command -v xdg-mime`
    if [ -n "${XDG_MIME}" ];then
        echo "Uninstalling mime type for checkboard test image..."
        ${XDG_MIME} uninstall mime-types/lsb-test-desktop-t2c-checkboard.xml
    else
        echo "Unable to find xdg-mime command..."
    fi
}

#---------------------------------------------------------

#remove previous results and temporaries
rm -rf results
rm -rf tet_tmp_dir

#install mime type
installmimetype

#run the tests
TSFNAME="tet_scen"

# a helper function
countTestGroups()
{
    # count the groups of tests to be executed (may be useful for the programs
    # that will be processing the execution log)
    NUM_TEST_GROUPS=0
    
    while read RAWLN
    do
        echo ${RAWLN} | grep ":include:.*-t2c\/.*scenarios\/" > /dev/null 
        if [ $? -eq 0 ]; then
            LINE_OK=`echo ${RAWLN} | LC_ALL=C sed 's/:include:[\/]*//'`
            
            if [ ! -f "${LINE_OK}" ]; then
                echo "File not found: ${PWD}/${LINE_OK}"
                exit 1
            fi
            
            NTG=`cat "${LINE_OK}" | grep "\/tests\/" | wc -l`
            NUM_TEST_GROUPS=`expr ${NUM_TEST_GROUPS} + ${NTG}`
        fi
    done < ${TSFNAME}

    echo "-------------------------------------"
    echo "Number of test groups to be executed: ${NUM_TEST_GROUPS}"
    echo "-------------------------------------"
} # end of countTestGroups()
    
if [ -z $1 ]
then
    countTestGroups
    tcc -e .
    
else
    #save old tet scenario file
    mv -f ${TSFNAME} ${TSFNAME}.orig
    
    echo "all" >> ${TSFNAME}

    cat ${TSFNAME}.orig | grep $1 >> ${TSFNAME}
    if [ $? != 0 ] 
    then 
        echo Test suite is not found: $1.
    else
        countTestGroups
                
        # run the tests
        tcc -e . 
    fi
    
    #restore old tet scenario file
    mv -f ${TSFNAME}.orig ${TSFNAME}
fi 

if [ ! -z "${USE_XVFB}" ]; then
    stopXvfb
fi

#remove the mime type
removemimetype

#remove temporaries
rm -rf tet_tmp_dir

exit 0
